@@ -191,6 +191,7 @@ get_legacy_settings (void)
 #endif
 	COPY_FROM_LEGACY (int,  "/gnome/desktop/peripherals/keyboard/bell_pitch",    "/Desktop/Bell/pitch=50");
 	COPY_FROM_LEGACY (int,  "/gnome/desktop/peripherals/keyboard/bell_duration", "/Desktop/Bell/duration=100");
+	g_object_unref (G_OBJECT (client));
 }
 
 #if 0
@@ -252,7 +253,7 @@ main (int argc, char **argv)
 			gtk_notebook_set_current_page (GTK_NOTEBOOK (WID ("keyboard_notebook")), 3);
 		}
 		capplet_set_icon (WID ("keyboard_dialog"),
-				  "keyboard-capplet.png");
+				  "gnome-dev-keyboard");
 		gtk_widget_show (WID ("keyboard_dialog"));
 		gtk_main ();
 	}
