@@ -59,8 +59,6 @@ const char GSWITCHIT_APPLET_CONFIG_KEY_ENABLED_PLUGINS[] =
 const char GSWITCHIT_APPLET_CONFIG_KEY_SECONDARIES[] =
     GSWITCHIT_APPLET_CONFIG_KEY_PREFIX "/secondary";
 
-static int gconfAppletListenerId = 0;
-
 /**
  * GSwitchItKbdConfig
  */
@@ -99,13 +97,11 @@ const char *GSWITCHIT_KBD_CONFIG_SYSBACKUP[] = {
 	GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_OPTIONS
 };
 
-static int gconfKbdListenerId;
-
 /**
  * static common functions
  */
 static void
-_GSwitchItKbdConfigListReset (GSList ** plist)
+_GSwitchItConfigStringListReset (GSList ** plist)
 {
 	while (*plist != NULL) {
 		GSList *p = *plist;
@@ -154,7 +150,7 @@ _GSwitchItKbdConfigGetDescriptions (const char *layoutName,
 }
 
 static void
-_GSwitchItKbdConfigAddListener (GConfClient * confClient,
+_GSwitchItConfigAddListener (GConfClient * confClient,
 			     const gchar * key,
 			     GConfClientNotifyFunc func,
 			     gpointer userData, int *pid)
@@ -171,7 +167,7 @@ _GSwitchItKbdConfigAddListener (GConfClient * confClient,
 }
 
 static void
-_GSwitchItKbdConfigRemoveListener (GConfClient * confClient, int *pid)
+_GSwitchItConfigRemoveListener (GConfClient * confClient, int *pid)
 {
 	if (*pid != 0) {
 		gconf_client_notify_remove (confClient, *pid);
@@ -272,8 +268,8 @@ _GSwitchItKbdConfigLayoutsAdd (GSwitchItKbdConfig * kbdConfig,
 }
 
 static void
-_GSwitchItKbdConfigCopyFromXklConfig (GSwitchItKbdConfig *
-				      kbdConfig, XklConfigRec * pdata)
+_GSwitchItKbdConfigCopyFromXklConfig (GSwitchItKbdConfig * kbdConfig,
+				      XklConfigRec * pdata)
 {
 	int i;
 	char **p, **p1;
@@ -323,8 +319,8 @@ _GSwitchItKbdConfigCopyFromXklConfig (GSwitchItKbdConfig *
 }
 
 static void
-_GSwitchItKbdConfigCopyToXklConfig (GSwitchItKbdConfig *
-				    kbdConfig, XklConfigRec * pdata)
+_GSwitchItKbdConfigCopyToXklConfig (GSwitchItKbdConfig * kbdConfig,
+				    XklConfigRec * pdata)
 {
 	int i;
 	pdata->model =
@@ -422,11 +418,13 @@ _GSwitchItKbdConfigLoadParams (GSwitchItKbdConfig * kbdConfig,
 			g_warning ("Error reading configuration:%s\n",
 				   gerror->message);
 			g_error_free (gerror);
+			g_free (pc);
 			gerror = NULL;
 		}
 		GSwitchItKbdConfigModelSet (kbdConfig, NULL);
 	} else {
 		GSwitchItKbdConfigModelSet (kbdConfig, pc);
+		g_free (pc);
 	}
 	XklDebug (150, "Loaded Kbd model: [%s]\n", kbdConfig->model);
 
@@ -449,6 +447,7 @@ _GSwitchItKbdConfigLoadParams (GSwitchItKbdConfig * kbdConfig,
 		_GSwitchItKbdConfigLayoutsAdd (kbdConfig, pl->data);
 		pl = pl->next;
 	}
+	_GSwitchItConfigStringListReset(&pl);
 
 	GSwitchItKbdConfigOptionsReset (kbdConfig);
 
@@ -470,6 +469,7 @@ _GSwitchItKbdConfigLoadParams (GSwitchItKbdConfig * kbdConfig,
 					       (const char *) pl->data);
 		pl = pl->next;
 	}
+	_GSwitchItConfigStringListReset(&pl);
 }
 
 static void
@@ -531,7 +531,7 @@ GSwitchItConfigTerm (GSwitchItConfig * config)
 }
 
 void
-GSwitchItConfigLoad (GSwitchItConfig * config)
+GSwitchItConfigLoadFromGConf (GSwitchItConfig * config)
 {
 	GError *gerror = NULL;
 
@@ -546,6 +546,7 @@ GSwitchItConfigLoad (GSwitchItConfig * config)
 		g_error_free (gerror);
 		gerror = NULL;
 	}
+	XklDebug (150, "groupPerApp: %d\n", config->groupPerApp);
 
 	config->handleIndicators =
 	    gconf_client_get_bool (config->confClient,
@@ -558,6 +559,7 @@ GSwitchItConfigLoad (GSwitchItConfig * config)
 		g_error_free (gerror);
 		gerror = NULL;
 	}
+	XklDebug (150, "handleIndicators: %d\n", config->handleIndicators);
 
 	config->layoutNamesAsGroupNames =
 	    gconf_client_get_bool (config->confClient,
@@ -570,6 +572,7 @@ GSwitchItConfigLoad (GSwitchItConfig * config)
 		g_error_free (gerror);
 		gerror = NULL;
 	}
+	XklDebug (150, "layoutNamesAsGroupNames: %d\n", config->layoutNamesAsGroupNames);
 
 	config->defaultGroup =
 	    gconf_client_get_int (config->confClient,
@@ -586,10 +589,11 @@ GSwitchItConfigLoad (GSwitchItConfig * config)
 	if (config->defaultGroup < -1
 	    || config->defaultGroup >= XklGetMaxNumGroups ())
 		config->defaultGroup = -1;
+	XklDebug (150, "defaultGroup: %d\n", config->defaultGroup);
 }
 
 void
-GSwitchItConfigSave (GSwitchItConfig * config)
+GSwitchItConfigSaveToGConf (GSwitchItConfig * config)
 {
 	GConfChangeSet *cs;
 	GError *gerror = NULL;
@@ -633,26 +637,42 @@ GSwitchItConfigActivate (GSwitchItConfig * config)
 }
 
 void
-GSwitchItConfigLockNextGroup ()
+GSwitchItConfigLockNextGroup (void)
 {
 	int group = XklGetNextGroup ();
 	XklLockGroup (group);
 }
 
 void
-GSwitchIttConfigLockPrevGroup ()
+GSwitchIttConfigLockPrevGroup (void)
 {
 	int group = XklGetPrevGroup ();
 	XklLockGroup (group);
 }
 
 void
-GSwitchItConfigRestoreGroup ()
+GSwitchItConfigRestoreGroup (void)
 {
 	int group = XklGetRestoreGroup ();
 	XklLockGroup (group);
 }
 
+void
+GSwitchItConfigStartListen (GSwitchItConfig * config,
+			    GConfClientNotifyFunc func,
+			    gpointer userData)
+{
+	_GSwitchItConfigAddListener (config->confClient,
+				     GSWITCHIT_CONFIG_DIR, func,
+				     userData, &config->configListenerId);
+}
+
+void
+GSwitchItConfigStopListen (GSwitchItConfig * config)
+{
+	_GSwitchItConfigRemoveListener (config->confClient,
+					&config->configListenerId);
+}
 /**
  * extern GSwitchItKbdConfig config functions
  */
@@ -688,7 +708,7 @@ GSwitchItKbdConfigTerm (GSwitchItKbdConfig * kbdConfig)
 }
 
 void
-GSwitchItKbdConfigLoad (GSwitchItKbdConfig * kbdConfig)
+GSwitchItKbdConfigLoadFromGConf (GSwitchItKbdConfig * kbdConfig)
 {
 	GError *gerror = NULL;
 
@@ -711,14 +731,14 @@ GSwitchItKbdConfigLoad (GSwitchItKbdConfig * kbdConfig)
 }
 
 void
-GSwitchItKbdConfigLoadSysBackup (GSwitchItKbdConfig * kbdConfig)
+GSwitchItKbdConfigLoadFromGConfBackup (GSwitchItKbdConfig * kbdConfig)
 {
 	_GSwitchItKbdConfigLoadParams (kbdConfig,
 				       GSWITCHIT_KBD_CONFIG_SYSBACKUP);
 }
 
 void
-GSwitchItKbdConfigLoadCurrent (GSwitchItKbdConfig * kbdConfig)
+GSwitchItKbdConfigLoadFromXCurrent (GSwitchItKbdConfig * kbdConfig)
 {
 	XklConfigRec data;
 	XklConfigRecInit (&data);
@@ -732,7 +752,7 @@ GSwitchItKbdConfigLoadCurrent (GSwitchItKbdConfig * kbdConfig)
 }
 
 void
-GSwitchItKbdConfigLoadInitial (GSwitchItKbdConfig * kbdConfig)
+GSwitchItKbdConfigLoadFromXInitial (GSwitchItKbdConfig * kbdConfig)
 {
 	XklConfigRec data;
 	XklConfigRecInit (&data);
@@ -761,7 +781,7 @@ GSwitchItKbdConfigEquals (GSwitchItKbdConfig * kbdConfig1,
 }
 
 void
-GSwitchItKbdConfigSave (GSwitchItKbdConfig * kbdConfig)
+GSwitchItKbdConfigSaveToGConf (GSwitchItKbdConfig * kbdConfig)
 {
 	GConfChangeSet *cs;
 	GError *gerror = NULL;
@@ -788,7 +808,7 @@ GSwitchItKbdConfigSave (GSwitchItKbdConfig * kbdConfig)
 }
 
 void
-GSwitchItKbdConfigSaveSysBackup (GSwitchItKbdConfig * kbdConfig)
+GSwitchItKbdConfigSaveToGConfBackup (GSwitchItKbdConfig * kbdConfig)
 {
 	GConfChangeSet *cs;
 	GError *gerror = NULL;
@@ -836,13 +856,13 @@ GSwitchItKbdConfigLayoutsAdd (GSwitchItKbdConfig * kbdConfig,
 void
 GSwitchItKbdConfigLayoutsReset (GSwitchItKbdConfig * kbdConfig)
 {
-	_GSwitchItKbdConfigListReset (&kbdConfig->layouts);
+	_GSwitchItConfigStringListReset (&kbdConfig->layouts);
 }
 
 void
 GSwitchItKbdConfigOptionsReset (GSwitchItKbdConfig * kbdConfig)
 {
-	_GSwitchItKbdConfigListReset (&kbdConfig->options);
+	_GSwitchItConfigStringListReset (&kbdConfig->options);
 }
 
 void
@@ -895,16 +915,16 @@ GSwitchItKbdConfigStartListen (GSwitchItKbdConfig * kbdConfig,
 			       GConfClientNotifyFunc func,
 			       gpointer userData)
 {
-	_GSwitchItKbdConfigAddListener (kbdConfig->confClient,
+	_GSwitchItConfigAddListener (kbdConfig->confClient,
 				     GSWITCHIT_KBD_CONFIG_DIR, func,
-				     userData, &gconfKbdListenerId);
+				     userData, &kbdConfig->configListenerId);
 }
 
 void
 GSwitchItKbdConfigStopListen (GSwitchItKbdConfig * kbdConfig)
 {
-	_GSwitchItKbdConfigRemoveListener (kbdConfig->confClient,
-					&gconfKbdListenerId);
+	_GSwitchItConfigRemoveListener (kbdConfig->confClient,
+					&kbdConfig->configListenerId);
 }
 
 /**
@@ -1076,7 +1096,7 @@ GSwitchItAppletConfigUpdateImages (GSwitchItAppletConfig * appletConfig,
 }
 
 void
-GSwitchItAppletConfigLoad (GSwitchItAppletConfig * appletConfig)
+GSwitchItAppletConfigLoadFromGConf (GSwitchItAppletConfig * appletConfig)
 {
 	GError *gerror = NULL;
 
@@ -1120,7 +1140,7 @@ GSwitchItAppletConfigLoad (GSwitchItAppletConfig * appletConfig)
 }
 
 void
-GSwitchItAppletConfigSave (GSwitchItAppletConfig * appletConfig)
+GSwitchItAppletConfigSaveToGConf (GSwitchItAppletConfig * appletConfig)
 {
 	GConfChangeSet *cs;
 	GError *gerror = NULL;
@@ -1160,16 +1180,16 @@ GSwitchItAppletConfigStartListen (GSwitchItAppletConfig * appletConfig,
 				  GConfClientNotifyFunc func,
 				  gpointer userData)
 {
-	_GSwitchItKbdConfigAddListener (appletConfig->confClient,
+	_GSwitchItConfigAddListener (appletConfig->confClient,
 				     GSWITCHIT_APPLET_CONFIG_DIR, func,
-				     userData, &gconfAppletListenerId);
+				     userData, &appletConfig->configListenerId);
 }
 
 void
 GSwitchItAppletConfigStopListen (GSwitchItAppletConfig * appletConfig)
 {
-	_GSwitchItKbdConfigRemoveListener (appletConfig->confClient,
-					&gconfAppletListenerId);
+	_GSwitchItConfigRemoveListener (appletConfig->confClient,
+					&appletConfig->configListenerId);
 }
 
 Bool
@@ -1230,10 +1250,24 @@ GSwitchItConfigLoadGroupDescriptionsUtf8 (GSwitchItConfig * config)
 					     (lDescr, vDescr), -1, NULL,
 					     NULL, NULL);
 					rv = g_slist_append (rv, nameUtf);
+				} else {
+					rv = g_slist_append (rv, g_strdup(""));
 				}
 			}
 		}
 		XklConfigRecDestroy (&xklConfig);
+		/* Worst case - multiple layous - but SOME of them are multigrouped :((( 
+		   We cannot do much - just add empty descriptions. 
+		   The UI is going to be messy. 
+		   Canadian layouts are famous for this sh.t. */
+		if (g_slist_length (rv) != XklGetNumGroups())
+		{
+			XklDebug (0, "The mismatch between "
+				  "the number of groups: %d and number of layouts: %d\n",
+				  XklGetNumGroups(), g_slist_length (rv));
+			for (i = XklGetNumGroups() - g_slist_length (rv); --i>=0;)
+				rv = g_slist_append (rv, g_strdup(""));
+		}
 	}
 	if (rv == NULL)
 		for (i = XklGetNumGroups (); --i >= 0;)
